      SUBROUTINE CALBCS(NMAX,X,Y,FCT,NX,NY,LAMBDA,MUE,NPNT,PNT,NCT,CT,
     &                  IERR)
C
CC  CALculate BiCubic Spline at given x-y grid coordinates
C
C************************ FFORM VERSION 3.0 ********** 31-JAN-90 09:20
C
CA  author : EXSAS              date: 24-NOV-1991 13:00
CU  update : EXSAS              date: 03-AUG-1993 16:48
C
CT  status: not tested
C
C   general description:
CC
CG  Subroutine CALBCS evaluates a two-dimensional spline function
CG  FCT(X,Y) of coordinates (X,Y), which is given in a bicubic spline
CG  representation.
CG  The arrays LAMBDA and MUE specify the complete set of knots
CG  associated with the variables X and Y, respectively.
CG
CG  The following checks will be done:
CG  - No point (X,Y) lies outside the rectangle defined by the
CG    extreme knot values.
CG  - each set of knots has to be in a non-decreasing order
CG  - NMAX > 0, NX and NY are each equal to at least 8,
CG    NCT = (NX-4)*(NY-4) and NPNT has to be large enough.
CG  - no knot has a multiplicity > 4
CG  - the data has to be ordered in panels as described above
CG
CG  If any of these checks fails, the subroutine exits immediately
CG  with an integer IERR > 2. On normal exit IERR equals zero.
CG
CG  References:
CG  De Boor, C., On calculating with B-splines
CG               J. Approx. Theory 6 (1972) 50-62
CG  Cox, M.G.,   The numerical evaluation of B-splines
CG               J. Instr. Maths Applics 10 (1972) 134-149
CG               NPL Division Report DNAC 4 (1971)
CG  Dierckx, P., An Algorithm for Surface-Fitting with Spline Functions
CG               IMA J. Numerical Analysis 1 (1981) 267-283
CG  Dierckx, P., An algorithm for surface fitting with spline functions
CG               Report TW50, Dept. Computer Sciences, K.U.Leuven (1980)
CG
C
C   call_var.          type I/O description
CP  NMAX                I   I/O maximum number of x,y coordinates
CP  X                   D   I/O x-coordinates of grid
CP  Y                   D   I/O y-coordinates of grid
CP  FCT                 D     O spline function
CP  NX                  I   I/O number of knots in x
CP  NY                  I   I/O number of knots in y
CP  LAMBDA              D   I/O array of knots in x-direction
CP  MUE                 D   I/O array of knots in y-direction
CP  NPNT                I   I/O number of points
CP  PNT                 I   I/O list of points
CP  NCT                 I   I/O number of b-spline coefficients
CP  CT                  D   I/O array with b-spline coefficients
CP  IERR                I     O = 0 no error 
CP                              = 3 one point (X,Y) lies outside the
CP                                  rectangle defined by the extreme
CP                                  knot values
CP                              = 4 one set of knots is not in a
CP                                  non-decreasing order
CP                              = 5 NMAX <= 0, NX and NY are not each
CP                                  equal to at least 8,
CP                                  NCT <> (NX-4)*(NY-4)
CP                                  NPNT is not large enough
CP                              = 6 knot has a multiplicity > 4
CP                              = 7 data is not ordered in panels
C
C   include_block_name          description
CI  RCOMMON:CGENL.CMN           general parameter common block
C
C   routines_called    type     description
CR
C
C   extensions/system calls     description
CX  
C
C***********************************************************************
C
C   variables   meaning
C
C***********************************************************************
C
      IMPLICIT NONE
C
      INTEGER         NMAX,NX,NY,NPNT,NCT,IERR
chb      DOUBLE PRECISION
chb     &            X(NMAX),Y(NMAX),FCT(NMAX),LAMBDA(NX),MUE(NY)
      double precision x(*),y(*),fct(*),lambda(*),mue(*)
      DOUBLE PRECISION
     &            CT(NCT)
      INTEGER         PNT(NPNT)
      DOUBLE PRECISION
     &            DF1,DF2,DF3,DF4,PD1,PD2,PD3,ONE,MXL,AUX,WI,XI,YI
      INTEGER         IADRES,IAUX,NXM4,NXM7,NYM4,NYM7,NP
      INTEGER         I,IX,IXINC,IXSAVE,IY,IYINC,I1,I2,I3
      DOUBLE PRECISION
     &            DX(6),DY(6),MX(4),MY(4),LA(6),MU(6)
      CHARACTER*6       RTNAME
C
      INCLUDE 'include/cgenl.cmn'
C
      SAVE
      DATA          RTNAME /'CALBCS'/
      DATA          ONE    /1.0D+0/
C
      IERR = 0
      NXM7 = NX - 7
      NYM7 = NY - 7
      NP   = NXM7*NYM7
      NXM4 = NX - 4
      NYM4 = NY - 4
      IF (IERR .EQ. 0) THEN
        LA(1) = LAMBDA(4)
        LA(2) = LAMBDA(NX-3)
        MU(1) = MUE(4)
        MU(2) = MUE(NY-3)
        DO I = 1, NMAX
          XI = X(I)
          YI = Y(I)
          IF (XI.LT.LA(1) .OR. XI.GT.LA(2) .OR.
     &        YI.LT.MU(1) .OR. YI.GT.MU(2)) IERR = 3
        END DO
      END IF
      IF (IERR .EQ. 0) THEN
        DO I = 2, NX
          IF (LAMBDA(I-1).GT.LAMBDA(I)) IERR = 4
        END DO
        DO I = 2, NY
          IF (MUE(I-1).GT.MUE(I)) IERR = 4
        END DO
      END IF
      IF (NMAX .LE. 0 .OR.
     &    NX .LT. 8 .OR. NY .LT. 8 .OR.
     &    NCT .NE. NXM4*NYM4 .OR.
     &    NPNT .LT. NMAX+NP) IERR = 5
      IF (IERR .EQ. 0) THEN
        DO I = 1, NXM4
          IF (LAMBDA(I).EQ.LAMBDA(I+4)) IERR = 6
        END DO
        DO I = 1, NYM4
           IF (MUE(I).EQ.MUE(I+4)) IERR = 6
        END DO
      END IF
      IF (IERR .EQ. 0) THEN
        IXSAVE = 0
        DO I = 1, NP
          IADRES = I + NMAX
          IF (PNT(IADRES).GT.0) THEN
            IX = (I-1)/NYM7
            IY = I - IX*NYM7
            IX = IX + 1
            IF (IX .NE. IXSAVE) THEN
              DO IXINC =1, 6
                LA(IXINC) = LAMBDA(IX+IXINC)
              END DO
              DX(1) = ONE/(LA(4)-LA(1))
              DX(2) = ONE/(LA(5)-LA(2))
              DX(3) = ONE/(LA(6)-LA(3))
              DX(4) = ONE/(LA(4)-LA(2))
              DX(5) = ONE/(LA(5)-LA(3))
              DX(6) = ONE/(LA(4)-LA(3))
              IXSAVE = IX
            END IF
            DO IYINC =1, 6
              MU(IYINC) = MUE(IY+IYINC)
            END DO
            DY(1) = ONE/(MU(4)-MU(1))
            DY(2) = ONE/(MU(5)-MU(2))
            DY(3) = ONE/(MU(6)-MU(3))
            DY(4) = ONE/(MU(4)-MU(2))
            DY(5) = ONE/(MU(5)-MU(3))
            DY(6) = ONE/(MU(4)-MU(3))
  100       IADRES = PNT(IADRES)
            IF (IADRES.GT.0) THEN
              XI = X(IADRES)
              YI = Y(IADRES)
              IF (XI .LT. LA(3) .OR. XI .GT. LA(4) .OR.
     &           (XI .EQ. LA(4) .AND. IX .NE. NXM7)
     &           .OR. YI .LT. MU(3) .OR. YI .GT. MU(4) .OR.
     &           (YI .EQ. MU(4) .AND. IY .NE. NYM7)) IERR = 7
              IF (IERR .EQ. 0) THEN
                DF1   = LA(5) - XI
                DF2   = LA(4) - XI
                DF3   = XI - LA(3)
                DF4   = XI - LA(2)
                PD2   = DF3*DX(5)
                PD1   = DF2*DX(4)
                PD3   = DF3*PD2*DX(3)
                PD2   = (DF4*PD1+DF1*PD2)*DX(2)
                PD1   = DF2*PD1*DX(1)
                WI    = DX(6)
                MX(4) = WI*DF3*PD3
                MX(3) = WI*(DF4*PD2+(LA(6)-XI)*PD3)
                MX(2) = WI*((XI-LA(1))*PD1+DF1*PD2)
                MX(1) = WI*DF2*PD1
                DF1   = MU(5) - YI
                DF2   = MU(4) - YI
                DF3   = YI - MU(3)
                DF4   = YI - MU(2)
                PD2   = DF3*DY(5)
                PD1   = DF2*DY(4)
                PD3   = DF3*PD2*DY(3)
                PD2   = (DF4*PD1+DF1*PD2)*DY(2)
                PD1   = DF2*PD1*DY(1)
                WI    = DY(6)
                MY(4) = WI*DF3*PD3
                MY(3) = WI*(DF4*PD2+(MU(6)-YI)*PD3)
                MY(2) = WI*((YI-MU(1))*PD1+DF1*PD2)
                MY(1) = WI*DF2*PD1
                I2 = (IX-1)*NYM4 + IY
                AUX = 0.D+0
                DO I3 = 1, 4
                  MXL = MX(I3)
                  DO I1 = 1, 4
                    IAUX = I2+I1-1
                    AUX  = AUX + MXL*MY(I1)*CT(IAUX)
                  END DO
                  I2 = I2 + NYM4
                END DO
                FCT(IADRES) = AUX
                GO TO 100
              END IF
            END IF
          END IF
        END DO
      END IF
      IF (IERR .NE. 0) THEN
        IF (IERR .EQ. 3) WRITE(OUTPT,1030)
        IF (IERR .EQ. 4) WRITE(OUTPT,1040)
        IF (IERR .EQ. 5) WRITE(OUTPT,1050)
        IF (IERR .EQ. 6) WRITE(OUTPT,1060)
        IF (IERR .EQ. 7) WRITE(OUTPT,1070)
        CALL WRERR(RTNAME,IERR,OUTPT)
 1030   FORMAT('At least one point (X,Y) lies outside rectangle.')
 1040   FORMAT('One set of knots is not in a non-decreasing order.')
 1050   FORMAT('NMAX <= 0, NX and NY <> 8, NCT <> (NX-4)*(NY-4),
     &          NPNT is not large enough')
 1060   FORMAT('At least one knot has a multiplicity > 4.')
 1070   FORMAT('Data is not ordered in panels.')
      END IF
      RETURN
      END     

